import { Authorized, Get, JsonController, OnUndefined, Param } from 'routing-controllers';
import { OpenAPI, ResponseSchema } from 'routing-controllers-openapi';
import { getConnectionManager, Repository } from 'typeorm';
import { RunnerCardNotFoundError } from '../errors/RunnerCardErrors';
import { RunnerCard } from '../models/entities/RunnerCard';
import { ResponseRunnerCard } from '../models/responses/ResponseRunnerCard';

@JsonController('/cards')
@OpenAPI({ security: [{ "AuthToken": [] }, { "RefreshTokenCookie": [] }] })
export class RunnerCardController {
	private cardRepository: Repository<RunnerCard>;

	/**
	 * Gets the repository of this controller's model/entity.
	 */
	constructor() {
		this.cardRepository = getConnectionManager().get().getRepository(RunnerCard);
	}

	@Get()
	@Authorized("CARD:GET")
	@ResponseSchema(ResponseRunnerCard, { isArray: true })
	@OpenAPI({ description: 'Lists all card.' })
	async getAll() {
		let responseCards: ResponseRunnerCard[] = new Array<ResponseRunnerCard>();
		const cards = await this.cardRepository.find({ relations: ['runner'] });
		cards.forEach(card => {
			responseCards.push(new ResponseRunnerCard(card));
		});
		return responseCards;
	}

	@Get('/:id')
	@Authorized("CARD:GET")
	@ResponseSchema(ResponseRunnerCard)
	@ResponseSchema(RunnerCardNotFoundError, { statusCode: 404 })
	@OnUndefined(RunnerCardNotFoundError)
	@OpenAPI({ description: "Lists all information about the card whose id got provided." })
	async getOne(@Param('id') id: number) {
		let card = await this.cardRepository.findOne({ id: id }, { relations: ['runner'] });
		if (!card) { throw new RunnerCardNotFoundError(); }
		return card.toResponse();
	}

	// @Post()
	// @Authorized("CARD:CREATE")
	// @ResponseSchema(ResponseTrack)
	// @ResponseSchema(TrackLapTimeCantBeNegativeError, { statusCode: 406 })
	// @OpenAPI({ description: "Create a new track. <br> Please remember that the track\'s distance must be greater than 0." })
	// async post(
	// 	@Body({ validate: true })
	// 	track: CreateTrack
	// ) {
	// 	return new ResponseTrack(await this.trackRepository.save(track.toTrack()));
	// }

	// @Put('/:id')
	// @Authorized("CARD:UPDATE")
	// @ResponseSchema(ResponseTrack)
	// @ResponseSchema(TrackNotFoundError, { statusCode: 404 })
	// @ResponseSchema(TrackIdsNotMatchingError, { statusCode: 406 })
	// @ResponseSchema(TrackLapTimeCantBeNegativeError, { statusCode: 406 })
	// @OpenAPI({ description: "Update the track whose id you provided. <br> Please remember that ids can't be changed." })
	// async put(@Param('id') id: number, @Body({ validate: true }) updateTrack: UpdateTrack) {
	// 	let oldTrack = await this.trackRepository.findOne({ id: id });

	// 	if (!oldTrack) {
	// 		throw new TrackNotFoundError();
	// 	}

	// 	if (oldTrack.id != updateTrack.id) {
	// 		throw new TrackIdsNotMatchingError();
	// 	}
	// 	await this.trackRepository.save(await updateTrack.updateTrack(oldTrack));

	// 	return new ResponseTrack(await this.trackRepository.findOne({ id: id }));
	// }

	// @Delete('/:id')
	// @Authorized("CARD:DELETE")
	// @ResponseSchema(ResponseTrack)
	// @ResponseSchema(ResponseEmpty, { statusCode: 204 })
	// @OnUndefined(204)
	// @OpenAPI({ description: "Delete the track whose id you provided. <br> If no track with this id exists it will just return 204(no content)." })
	// async remove(@Param("id") id: number, @QueryParam("force") force: boolean) {
	// 	let track = await this.trackRepository.findOne({ id: id });
	// 	if (!track) { return null; }

	// 	const trackStations = (await this.trackRepository.findOne({ id: id }, { relations: ["stations"] })).stations;
	// 	if (trackStations.length != 0 && !force) {
	// 		throw new TrackHasScanStationsError();
	// 	}
	// 	const scanController = new ScanStationController;
	// 	for (let station of trackStations) {
	// 		scanController.remove(station.id, force);
	// 	}

	// 	await this.trackRepository.delete(track);
	// 	return new ResponseTrack(track);
	// }
}